"""English translations of the program interface"""

MSG_WELCOME_1 = "Welcome to Calcure"
MSG_WELCOME_2 = "Your terminal calendar and task manager!"
MSG_WELCOME_3 = "Config and data files were created at:"
MSG_WELCOME_4 = "For support, contribution, and additional information, visit:"
MSG_WELCOME_5 = "Press ? to see all keybindings or any other key to continue."

TITLE_KEYS_GENERAL = "GENERAL KEYBINDINGS"
TITLE_KEYS_CALENDAR = "CALENDAR KEYBINDINGS"
TITLE_KEYS_JOURNAL  = "JOURNAL KEYBINDINGS"

KEYS_GENERAL = {
        " Space ": "Switch between calendar and journal",
        "   /   ": "Toggle split screen",
        "   *   ": "Toggle global privacy",
        "   ?   ": "Toggle this help",
        "   q   ": "Quit",
        }

KEYS_CALENDAR = {
        "  a(A) ": "Add a (recurring) event",
        "  n,🠒  ": "Next month (day)",
        "  p,🠐  ": "Previous month (day)",
        "  d,x  ": "Delete an event",
        "  e,c  ": "Edit an event",
        "   m   ": "Move event",
        "   g   ": "Go to a certain day",
        "   h   ": "Toggle event as high priority",
        "   l   ": "Toggle event as low priority",
        "   .   ": "Toggle event privacy",
        "   C   ": "Import events from calcurse",
        "   G   ": "Return to current month (day)",
        }

KEYS_TODO = {
        "  a(A) ": "Add new (sub)task",
        "  h(H) ": "Toggle one (all) of the tasks as high priority",
        "  l(L) ": "Toggle one (all) of the tasks as low priority",
        "  v(V) ": "Toggle one (all) of the tasks as done",
        "  u(U) ": "Unmark one (all) of the tasks",
        "  d(D) ": "Delete one (all) of the tasks (with all subtasks)",
        "  t(T) ": "Start/pause (remove) timer for a task",
        "  e,c  ": "Edit a task",
        "   s   ": "Toggle between task and subtask",
        "   .   ": "Toggle task privacy",
        "  f(F) ": "Change (remove) task deadline",
        "   m   ": "Move a task",
        "  C(W) ": "Import tasks from calcurse (taskwarrior)",
        }

MSG_NAME          = "CALCURE"
MSG_VIM           = "Vim keys (j, k, ZZ, ZQ) work as well!"
MSG_INFO          = "For more information, visit:"
MSG_SITE          = "https://github.com/anufrievroman/calcure"
MSG_EXIT          = "Really exit? (y/n) "

MSG_EVENT_HIGH    = "Mark as high priority event number: "
MSG_EVENT_LOW     = "Mark as low priority event number: "
MSG_EVENT_RESET   = "Reset status for event number: "
MSG_EVENT_DEL     = "Delete event number: "
MSG_EVENT_REN     = "Rename event number: "
MSG_NEW_TITLE     = "Enter new title: "
MSG_EVENT_MOVE    = "Move event number: "
MSG_EVENT_MOVE_TO = "Move event to: "
MSG_EVENT_DATE    = "Enter date: "
MSG_EVENT_TITLE   = "Enter title: "
MSG_EVENT_REP     = "How many times repeat the event: "
MSG_EVENT_FR      = "Repeat the event every (d)ay, (w)eek, (m)onth or (y)ear? "
MSG_EVENT_IMP     = "Import events from Calcurse? (y/n)"
MSG_EVENT_PRIVACY = "Toggle privacy of event number: "
MSG_TM_ADD        = "Add/pause timer for task number: "
MSG_TM_RESET      = "Remove timer for the task number: "
MSG_TS_HIGH       = "Mark as high priority the task number: "
MSG_TS_LOW        = "Mark as low priority the task number: "
MSG_TS_RES        = "Reset status for the task number: "
MSG_TS_DONE       = "Mark as done the task number: "
MSG_TS_DEL        = "Delete task number: "
MSG_TS_DEL_ALL    = "Really delete all tasks? (y/n)"
MSG_TS_MOVE       = "Move task from number: "
MSG_TS_MOVE_TO    = "Move task to number: "
MSG_TS_EDIT       = "Edit task number: "
MSG_TS_TOG        = "Toggle subtask number: "
MSG_TS_SUB        = "Add subtask for task number: "
MSG_TS_TITLE      = "Enter subtask: "
MSG_TS_IM         = "Import tasks from Calcurse? (y/n)"
MSG_TS_TW         = "Import tasks from Taskwarrior? (y/n)"
MSG_TS_NOTHING    = "Nothing planned..."
MSG_TS_PRIVACY    = "Toggle privacy of task number: "
MSG_TS_DEAD_ADD   = "Add deadline for task number: "
MSG_TS_DEAD_DEL   = "Remove deadline of the task number: "
MSG_TS_DEAD_DATE  = "Add deadline on (YYYY/MM/DD): "
MSG_WEATHER       = "Weather is loading..."

CALENDAR_HINT     = "Space · Switch to journal   a · Add event  n/p · Change month   ? · All keybindings"
CALENDAR_HINT_D   = "Space · Switch to journal   a · Add event  n/p · Change day   ? · All keybindings"
JOURNAL_HINT      = "Space · Switch to calendar   a · Add task   v · Done   i · Important   ? · All keybindings"

DAYS = ["MONDAY", "TUESDAY", "WEDNESDAY", "THURSDAY", "FRIDAY", "SATURDAY", "SUNDAY"]
DAYS_PERSIAN = ["SHANBEH", "YEKSHANBEH", "DOSHANBEH", "SESHANBEH", "CHAHARSHANBEH", "PANJSHANBEH", "JOMEH"]

MONTHS = ["JANUARY", "FEBRUARY", "MARCH", "APRIL", "MAY", "JUNE", "JULY", "AUGUST", "SEPTEMBER", "OCTOBER", "NOVEMBER", "DECEMBER"]
MONTHS_PERSIAN = ["FARVARDIN", "ORDIBEHESHT", "KHORDAD", "TIR", "MORDAD", "SHAHRIVAR", "MEHR", "ABAN", "AZAR", "DEY", "BAHMAN", "ESFAND"]



"""Module provides datatypes used in the program"""

import time
import enum

from calcure.calendars import Calendar


class AppState(enum.Enum):
    """Possible focus states of the application"""
    CALENDAR = 1
    JOURNAL = 2
    HELP = 3
    EXIT = 4
    WELCOME = 5


class CalState(enum.Enum):
    """Possible states of the calendar view"""
    MONTHLY = 1
    DAILY = 2


class Status(enum.Enum):
    """Status of events and tasks"""
    NORMAL = 1
    DONE = 2
    IMPORTANT = 3
    UNIMPORTANT = 4


class Frequency(enum.Enum):
    """Frequency of repetitions of recurring events"""
    ONCE = 1
    DAILY = 2
    WEEKLY = 3
    MONTHLY = 4
    YEARLY = 5


class Color(enum.Enum):
    """Colors read from user config"""
    DAY_NAMES = 1
    WEEKENDS = 2
    HINTS = 3
    TODAY = 4
    DAYS = 5
    WEEKEND_NAMES = 6
    BIRTHDAYS = 7
    PROMPTS = 8
    CONFIRMATIONS = 9
    TITLE = 10
    TODO = 11
    DONE = 12
    IMPORTANT = 13
    TIMER = 14
    TIMER_PAUSED = 15
    HOLIDAYS = 16
    EVENTS = 17
    TIME = 18
    WEATHER = 19
    UNIMPORTANT = 20
    CALENDAR_HEADER = 21
    ACTIVE_PANE = 22
    SEPARATOR = 23
    EMPTY = 24
    CALENDAR_BOARDER = 24
    DEADLINES = 25


class Task:
    """Tasks crated by user"""

    def __init__(self, item_id, name, status, timer, privacy, year=0, month=0, day=0):
        self.item_id = item_id
        self.name = name
        self.status = status
        self.timer = timer
        self.privacy = privacy
        self.year = year
        self.month = month
        self.day = day


class Event:
    """Parent class of events"""

    def __init__(self, year, month, day, name):
        self.year = year
        self.month = month
        self.day = day
        self.name = name


class UserEvent(Event):
    """Events crated by user"""

    def __init__(self, item_id, year, month, day, name, repetition, frequency, status, privacy):
        super().__init__(year, month, day, name)
        self.item_id = item_id
        self.repetition = repetition
        self.frequency = frequency
        self.status = status
        self.privacy = privacy


class UserRepeatedEvent(Event):
    """Events that are repetitions of the original user events"""

    def __init__(self, item_id, year, month, day, name, status, privacy):
        super().__init__(year, month, day, name)
        self.item_id = item_id
        self.status = status
        self.privacy = privacy


class Timer:
    """Timer for tasks"""

    def __init__(self, stamps):
        self.stamps = stamps

    @property
    def is_counting(self):
        """Evaluate if the timer is currently running"""
        return False if not self.stamps else (len(self.stamps)%2 == 1)

    @property
    def is_started(self):
        """Evaluate whether the timer has started"""
        return True if self.stamps else False

    @property
    def passed_time(self):
        """Calculate how much time has passed in the un-paused intervals"""
        time_passed = 0

        # Calculate passed time, assuming that even timestamps are pauses:
        for index, _ in enumerate(self.stamps):
            if index > 0 and index % 2 == 1:
                time_passed += float(self.stamps[index]) - float(self.stamps[index-1])

        # Add time passed during the current run:
        if self.is_counting:
            time_passed += time.time() - float(self.stamps[-1])

        # Depending on how much time has passed, show in different formats:
        one_hour = 60*60.0
        one_day = 24*one_hour
        if time_passed < one_hour:
            format_string = "%M:%S"
        else:
            format_string = "%H:%M:%S"
        time_string = str(time.strftime(format_string, time.gmtime(int(time_passed))))

        if 2*one_day > time_passed > one_day:
            time_string = "1 day " + time_string
        if time_passed >= 2*one_day:
            time_string = str(int(time_passed//one_day)) + " days " + time_string
        return time_string


class Collection:
    """Parent class for collections of items like tasks or events"""

    def __init__(self):
        self.items = []
        self.changed = False

    def add_item(self, item):
        """Add an item to the collection"""
        if 100 > len(item.name) > 0 and item.name != "\[":
            self.items.append(item)
            self.changed = True

    def delete_item(self, selected_task_id):
        """Delete an item with provided id from the collection"""
        for item in self.items:
            if item.item_id == selected_task_id:
                self.items.remove(item)
                self.changed = True
                break

    def rename_item(self, selected_task_id, new_name):
        """Edit an item name in the collection"""
        for item in self.items:
            if item.item_id == selected_task_id and len(new_name) > 0:
                item.name = new_name
                self.changed = True

    def toggle_item_status(self, selected_task_id, new_status):
        """Toggle the status for the item with provided id"""
        for item in self.items:
            if item.item_id == selected_task_id:
                if item.status == new_status:
                    item.status = Status.NORMAL
                else:
                    item.status = new_status
                self.changed = True
                break

    def toggle_item_privacy(self, selected_task_id):
        """Toggle the privacy for the item with provided id"""
        for item in self.items:
            if item.item_id == selected_task_id:
                item.privacy = not item.privacy
                self.changed = True
                break

    def item_exists(self, item_name):
        """Check if such item already exists in collection"""
        for item in self.items:
            if item.name == item_name:
                return True
        return False

    def change_all_statuses(self, new_status):
        """Change statuses of all items"""
        for item in self.items:
            item.status = new_status
            self.changed = True

    def delete_all_items(self):
        """Delete all items from the collection"""
        self.items.clear()
        self.changed = True

    def is_empty(self):
        """Check if the collection is empty"""
        return len(self.items) == 0

    def is_valid_number(self, number):
        """Check if input is valid and corresponds to an item"""
        if number is None:
            return False
        return 0 <= number < len(self.items)

    def filter_events_that_day(self, screen):
        """Filter only events that happen on the particular day"""
        events_of_the_day = Events()
        for event in self.items:
            if (event.year == screen.year
                and event.month == screen.month
                and event.day == screen.day):
                events_of_the_day.add_item(event)
        return events_of_the_day

    def filter_events_that_month(self, screen):
        """Filter only events that happen on the particular month and sort them by day"""
        events_of_the_month = Events()
        for event in self.items:
            if event.month == screen.month and event.year == screen.year:
                events_of_the_month.add_item(event)
        events_of_the_month.items = sorted(events_of_the_month.items, key=lambda item: item.day)
        return events_of_the_month


class Tasks(Collection):
    """List of tasks created by the user"""

    def add_subtask(self, task, number):
        """Add a subtask for certain task in the journal"""
        level = '----'if (self.items[number].name[:2] == '--') else '--'
        task.name = level + task.name
        if 100 > len(task.name) > 0:
            self.items.insert(number+1, task)
            self.changed = True

    def add_timestamp_for_task(self, selected_task_id):
        """Add a timestamp to this task"""
        for item in self.items:
            if item.item_id == selected_task_id:
                item.timer.stamps.append(int(time.time()))
                self.changed = True
                break

    def reset_timer_for_task(self, selected_task_id):
        """Reset the timer for one of the tasks"""
        for item in self.items:
            if item.item_id == selected_task_id:
                item.timer.stamps = []
                self.changed = True
                break

    def change_deadline(self, selected_task_id, new_year, new_month, new_day):
        """Reset the timer for one of the tasks"""
        for item in self.items:
            if item.item_id == selected_task_id:
                item.year = new_year
                item.month = new_month
                item.day = new_day
                self.changed = True
                break

    def toggle_subtask_state(self, selected_task_id):
        """Toggle the state of the task-subtask"""
        for item in self.items:
            if item.item_id == selected_task_id:
                if item.name[:2] == '--':
                    item.name = item.name[2:]
                else:
                    item.name = '--' + item.name
                self.changed = True

    def move_task(self, number_from, number_to):
        """Move task from certain place to another in the list"""
        self.items.insert(number_to, self.items.pop(number_from))
        self.changed = True

    def generate_id(self):
        """Generate a id for a new item. The id is generated as maximum of existing ids plus one"""
        if self.is_empty():
            return 0
        return max([item.item_id for item in self.items]) + 1


class Events(Collection):
    """List of events created by the user or imported"""

    def event_exists(self, new_event):
        """Check if such event already exists in collection"""
        for event in self.items:
            if (event.name == new_event.name
                and event.year == new_event.year
                and event.month == new_event.month
                and event.day == new_event.day):
                return True
        return False

    def change_day(self, selected_item_id, new_day):
        """Move an event to another day"""
        for item in self.items:
            if item.item_id == selected_item_id:
                item.day = new_day
                self.changed = True
                break


class Birthdays(Events):
    """List of birthdays imported from abook"""

    def filter_events_that_day(self, screen):
        """Filter only birthdays that happen on the particular day"""
        events_of_the_day = Events()
        for event in self.items:
            if event.month == screen.month and event.day == screen.day:
                events_of_the_day.add_item(event)
        return events_of_the_day


class RepeatedEvents(Events):
    """List of events that are repetitions of main events"""

    def __init__(self, user_events, use_persian_calendar):
        super().__init__()
        self.user_events = user_events
        self.use_persian_calendar = use_persian_calendar

        for event in self.user_events.items:
            if event.repetition >= 1:
                for rep in range(1, event.repetition):
                    temp_year = event.year + rep*(event.frequency == Frequency.YEARLY)
                    temp_month = event.month + rep*(event.frequency == Frequency.MONTHLY)
                    temp_day = event.day + rep*(event.frequency == Frequency.DAILY) + 7*rep*(event.frequency == Frequency.WEEKLY)
                    year, month, day = self.calculate_recurring_events(temp_year, temp_month, temp_day, event.frequency)
                    self.add_item(UserRepeatedEvent(event.item_id, year, month, day, event.name, event.status, event.privacy))

    def calculate_recurring_events(self, year, month, day, frequency):
        """Calculate the date of recurring events so that they occur in the next month or year"""
        new_day = day
        new_month = month
        new_year = year
        skip_days = 0

        # Weekly and daily recurrence:
        if frequency in [Frequency.WEEKLY, Frequency.DAILY]:

            # Calculate how many days and month to skip to next event:
            for i in range(1000):
                if month + i > 12:
                    year = year + 1
                    month = month - 12

                last_day = Calendar(0, self.use_persian_calendar).last_day(year, month+i)
                if day > skip_days + last_day:
                    skip_days += last_day
                    skip_months = i + 1
                else:
                    skip_months = i
                    break
            new_day = day - skip_days
            new_month = month + skip_months
            new_year = year

        # Monthly recurrence:
        if frequency == Frequency.MONTHLY:
            if month > 12:
                new_year = year + (month - 1)//12
                new_month = month - 12*(new_year - year)
        return new_year, new_month, new_day



"""Russian translations of the program interface"""

MSG_WELCOME_1 = "Добро пожаловать в Calcure"
MSG_WELCOME_2 = "Ваш новый календарь и менеджер задач!"
MSG_WELCOME_3 = "Файлы конфигурации и данных созданы в:"
MSG_WELCOME_4 = "Информация об участии, поддержке, и функционале доступна в репозитории:"
MSG_WELCOME_5 = "Нажмите ? для вызова списка клавиш или любую клавишу для продолжения."

TITLE_KEYS_GENERAL  = "ОБЩИЕ КЛАВИШИ"
TITLE_KEYS_CALENDAR = "КЛАВИШИ КАЛЕНДАРЯ"
TITLE_KEYS_JOURNAL  = "КЛАВИШИ ЖУРНАЛА"

KEYS_GENERAL = {
        " Space ": "Переключать между календарём и журналом",
        "   /   ": "Переключать режим разделённого экрана",
        "   *   ": "Переключать приватность",
        "   ?   ": "Вызвать экран помощи",
        "   q   ": "Выйти",
        }

KEYS_CALENDAR = {
        "  a(A) ": "Добавить (повторяющееся) событие",
        "  n,🠒  ": "Следующий месяц (день)",
        "  p,🠐  ": "Предыдущий месяц (день)",
        "  d,x  ": "Удалить событие",
        "  e,c  ": "Переименовать событие",
        "   m   ": "Переместить событие",
        "   g   ": "Открыть определённый день",
        "   h   ": "Переключать высокий приоритет",
        "   l   ": "Переключать низкий приоритет",
        "   .   ": "Переключать приватность события",
        "   C   ": "Импортировать события из calcurse",
        "   G   ": "Вернуться к текущему месяцу (дню)",
        }

KEYS_TODO = {
        "  a(A) ": "Добавить новую (под)задачу",
        "  h(H) ": "Переключать приоритет одной (всех) задач как высокий",
        "  l(L) ": "Переключать приоритет одной (всех) задач как низкий",
        "  v(V) ": "Переключать приоритет одной (всех) задач как завершено",
        "  u(U) ": "Убрать все метки с одной (всех) задач",
        "  d(D) ": "Удалить одну (все) задачи (со всеми подзадачами)",
        "  t(T) ": "Запустить/приостановить (удалить) таймер задачи",
        "  e,c  ": "Переименовать задачу",
        "   s   ": "Переключать между задачей и подзадачей",
        "   .   ": "Переключать приватность задачи",
        "  f(F) ": "Изменить (удалить) дедлайн задачи",
        "   m   ": "Переместить задачу",
        "  C(W) ": "Импортировать задачи из calcurse (taskwarrior)",
        }

MSG_NAME          = "CALCURE"
MSG_VIM           = "Vim клавиши (j, k, ZZ, ZQ) работают!"
MSG_INFO          = "Больше информации вы найдёте на:"
MSG_SITE          = "https://github.com/anufrievroman/calcure"
MSG_EXIT          = "Действительно выйти? (y/n) "

MSG_EVENT_HIGH    = "Отметить как высокоприоритетное событие номер: "
MSG_EVENT_LOW     = "Отметить как низкоприоритетное событие номер: "
MSG_EVENT_RESET   = "Обнулить статус событие номер: "
MSG_EVENT_DEL     = "Удалить событие номер: "
MSG_EVENT_REN     = "Переименовать событие номер: "
MSG_NEW_TITLE     = "Введите новое название: "
MSG_EVENT_MOVE    = "Переместить событие номер: "
MSG_EVENT_MOVE_TO = "Переместить событие на: "
MSG_EVENT_DATE    = "Введите дату: "
MSG_EVENT_TITLE   = "Введите название: "
MSG_EVENT_REP     = "Сколько раз повторить событие: "
MSG_EVENT_FR      = "Повторить событие каждый день (d), неделю (w), месяц (m) или год(y)? "
MSG_EVENT_IMP     = "Импортировать события из Calcurse? (y/n)"
MSG_EVENT_PRIVACY = "Изменить приватность события номер: "
MSG_TM_ADD        = "Запустить/приостановить таймер для задачи: "
MSG_TM_RESET      = "Удалить таймер для задачи номер: "
MSG_TS_HIGH       = "Установить высокий приоритет для задачи номер: "
MSG_TS_LOW        = "Установить низкий приоритет для задачи номер: "
MSG_TS_RES        = "Обнулить статус задачи номер: "
MSG_TS_DONE       = "Отметить как завершенную задачу номер: "
MSG_TS_DEL        = "Удалить задачу номер: "
MSG_TS_DEL_ALL    = "Действительно удалить все задачи? (y/n)"
MSG_TS_MOVE       = "Переместить задачу номер: "
MSG_TS_MOVE_TO    = "Переместить задачу на номер: "
MSG_TS_EDIT       = "Переименовать задачу номер: "
MSG_TS_TOG        = "Переключить уровень задачи номер: "
MSG_TS_SUB        = "Добавить подзадачу к задачу номер: "
MSG_TS_TITLE      = "Введите название подзадачи: "
MSG_TS_IM         = "Импортировать задачи из Calcurse? (y/n)"
MSG_TS_TW         = "Импортировать задачи из Taskwarrior? (y/n)"
MSG_TS_NOTHING    = "Ничего не запланировано..."
MSG_TS_PRIVACY    = "Переключать приватность задачи номер: "
MSG_TS_DEAD_ADD   = "Добавить дедлайн к задаче номер: "
MSG_TS_DEAD_DEL   = "Удалить дедлайн задачи номер: "
MSG_TS_DEAD_DATE  = "Установить дедлайн на (YYYY/MM/DD): "
MSG_WEATHER       = "Загружается информации о погоде..."

CALENDAR_HINT     = "Пробел · Переключить на журнал   a · Новое событие  n/p · Сменить месяц   ? · Клавиши"
CALENDAR_HINT_D   = "Пробел · Переключить на журнал   a · Новое событие  n/p · Сменить день   ? · Клавиши"
JOURNAL_HINT      = "Пробел · Переключить на календарь   a · Новая задача   v · Выполнено   h · Важно   ? · Клавиши"

DAYS         = ["ПОНЕДЕЛЬНИК", "ВТОРНИК", "СРЕДА", "ЧЕТВЕРГ", "ПЯТНИЦА", "СУББОТА", "ВОСКРЕСЕНЬЕ"]
DAYS_PERSIAN = ["SHANBEH", "YEKSHANBEH", "DOSHANBEH", "SESHANBEH", "CHAHARSHANBEH", "PANJSHANBEH", "JOMEH"]

MONTHS         = ["ЯНВАРЬ", "ФЕВРАЛЬ", "МАРТ", "АПРЕЛЬ", "МАЙ", "ИЮНЬ", "ИЮЛЬ", "АВГУСТ", "СЕНТЯБРЬ", "ОКТЯБРЬ", "НОЯБРЬ", "ДЕКАБРЬ"]
MONTHS_PERSIAN = ["FARVARDIN", "ORDIBEHESHT", "KHORDAD", "TIR", "MORDAD", "SHAHRIVAR", "MEHR", "ABAN", "AZAR", "DEY", "BAHMAN", "ESFAND"]
